@@ -14,7 +14,7 @@ __maintainer__ = "Giridhar Pemmasani (pgiri@yahoo.com)"
 __license__ = "MIT"
 __url__ = "http://dispy.sourceforge.net"
 __status__ = "Production"
-__version__ = "3.16"
+__version__ = "3.17"
 
 __all__ = ['logger', 'DispyJob', 'JobCluster', 'SharedJobCluster']
 
@@ -189,10 +189,11 @@ class _Compute(object):
         self.code = None
         self.dest_path = None
         self.xfer_files = []
-        self.cleanup = True
         self.nodes = {}
         self.node_spec = None
         self.reentrant = False
+        self.setup = None
+        self.cleanup = None
         self.scheduler_ip_addr = None
         self.scheduler_port = None
         self.scheduler_auth = None
@@ -267,7 +268,13 @@ class _Node(object):
         else:
             logger.warning('Got "%s": Ignoring node %s', resp, self.ip_addr)
             raise StopIteration(-1)
-        yield 0
+
+        if isinstance(compute.setup, str):
+            resp = yield self.send(b'SETUP:' + serialize(compute.id), coro=coro)
+            logger.debug('setup resp: %s' % resp)
+            if resp != b'ACK':
+                raise StopIteration(-1)
+        raise StopIteration(0)
 
     def send(self, msg, reply=True, coro=None):
         # generator
@@ -325,8 +332,7 @@ class _Node(object):
 
     def close(self, compute, coro=None):
         # generator
-        logger.debug('Closing node %s for %s / %s / %s', self.ip_addr, compute.name, compute.id,
-                     compute.cleanup)
+        logger.debug('Closing node %s for %s / %s', self.ip_addr, compute.name, compute.id)
         msg = b'DEL_COMPUTE:' + serialize({'ID':compute.id})
         try:
             v = yield self.send(msg, reply=False, coro=coro)
@@ -471,12 +477,16 @@ class _Cluster(object):
                         self.ext_ip_addrs.add(addr)
                     else:
                         logger.warning('ignoring invalid ext_ip_addr "%s"' % node)
-            if not port:
+            if port:
+                port = int(port)
+            else:
                 if shared:
                     port = 0
                 else:
                     port = 51347
-            if not node_port:
+            if node_port:
+                node_port = int(node_port)
+            else:
                 node_port = 51348
 
             self.port = port
@@ -1406,7 +1416,7 @@ class JobCluster(object):
 
     def __init__(self, computation, nodes=None, depends=[], callback=None,
                  ip_addr=None, port=None, node_port=None, ext_ip_addr=None,
-                 dest_path=None, loglevel=logging.INFO, cleanup=True,
+                 dest_path=None, loglevel=logging.INFO, setup=None, cleanup=True,
                  ping_interval=None, pulse_interval=None, poll_interval=None,
                  reentrant=False, secret='', keyfile=None, certfile=None, fault_recover=None):
         """Create an instance of cluster for a specific computation.
@@ -1577,6 +1587,15 @@ class JobCluster(object):
         else:
             shared = False
 
+        if setup:
+            assert inspect.isfunction(setup), "setup must be Python function"
+            depends.append(setup)
+
+        if cleanup:
+            if cleanup is not True:
+                assert inspect.isfunction(cleanup), "cleanup must be Python function"
+                depends.append(cleanup)
+
         if fault_recover:
             if fault_recover is True:
                 now = datetime.datetime.now()
@@ -1606,8 +1625,8 @@ class JobCluster(object):
             logger.info('Storing fault recovery information in "%s"', self.fault_recover_file)
 
         self._cluster = _Cluster(ip_addr=ip_addr, port=port, node_port=node_port,
-                                 ext_ip_addr=ext_ip_addr, shared=shared, secret=secret,
-                                 keyfile=keyfile, certfile=certfile)
+                                 ext_ip_addr=ext_ip_addr, shared=shared,
+                                 secret=secret, keyfile=keyfile, certfile=certfile)
         atexit.register(self.shutdown)
         # self.ip_addr = self._cluster.ip_addr
 
@@ -1678,19 +1697,21 @@ class JobCluster(object):
                 logger.warning('dest_path must not be absolute path')
             dest_path = dest_path.lstrip(os.sep)
             compute.dest_path = dest_path
-            compute.cleanup = (cleanup == True)
-        else:
-            if not cleanup:
-                logger.warning('"cleanup" argument is ignored if dest_path is not given')
-            compute.cleanup = True
 
         compute.scheduler_port = self._cluster.port
         compute.scheduler_auth = self._cluster.auth_code
         compute.job_result_port = self._cluster.port
-        compute.cleanup = cleanup
         compute.reentrant = reentrant
         compute.pulse_interval = pulse_interval
         compute.poll_interval = poll_interval
+        if inspect.isfunction(setup):
+            compute.setup = setup.__name__
+        else:
+            compute.setup = None
+        if inspect.isfunction(cleanup):
+            compute.cleanup = cleanup.__name__
+        else:
+            compute.cleanup = cleanup
 
         if not shared:
             compute.node_spec = _parse_nodes(nodes)
@@ -1793,7 +1814,7 @@ class SharedJobCluster(JobCluster):
     """
     def __init__(self, computation, nodes=None, depends=[], callback=None,
                  ip_addr=None, port=None, scheduler_node=None, scheduler_port=None,
-                 ext_ip_addr=None, loglevel=logging.INFO, cleanup=True,
+                 ext_ip_addr=None, loglevel=logging.INFO, setup=None, cleanup=True,
                  pulse_interval=None, ping_interval=None, poll_interval=None,
                  reentrant=False, secret='', keyfile=None, certfile=None, fault_recover=None):
 
@@ -1816,7 +1837,7 @@ class SharedJobCluster(JobCluster):
 
         JobCluster.__init__(self, computation, nodes='dummy', depends=depends,
                             callback=callback, ip_addr=ip_addr, port=port, ext_ip_addr=ext_ip_addr,
-                            loglevel=loglevel, cleanup=cleanup, pulse_interval=None,
+                            loglevel=loglevel, setup=setup, cleanup=cleanup, pulse_interval=None,
                             poll_interval=None, reentrant=reentrant, fault_recover=fault_recover)
         if pulse_interval is not None:
             logger.warning('pulse_interval is not used in SharedJobCluster; ' \
