@@ -607,6 +607,28 @@ class _DispyNode(object):
                     logger.debug('Could not send reply for "%s"', xf.name)
             raise StopIteration # xfer_file_task
 
+        def setup_computation(msg):
+            try:
+                compute_id = unserialize(msg)
+                compute = self.computations[compute_id]
+                assert isinstance(compute.setup, str)
+                os.chdir(compute.dest_path)
+                exec(marshal.loads(compute.code)) in globals(), locals()
+                _dispy_setup_func = locals()[compute.setup]
+                assert _dispy_setup_func() == 0
+            except:
+                logger.debug('setup "%s" failed' % compute.setup)
+                logger.debug(traceback.format_exc())
+                resp = b'NAK (setup failed)'
+            else:
+                resp = b'ACK'
+            if resp != b'ACK':
+                if not compute.cleanup:
+                    compute.cleanup = True
+                compute.zombie = True
+                self.cleanup_computation(compute)
+            yield conn.send_msg(resp)
+
         def terminate_job_task(msg):
             try:
                 _job = unserialize(msg)
@@ -822,6 +844,10 @@ class _DispyNode(object):
             msg = msg[len(b'FILEXFER:'):]
             yield xfer_file_task(msg)
             conn.close()
+        elif msg.startswith(b'SETUP:'):
+            msg = msg[len(b'SETUP:'):]
+            yield setup_computation(msg)
+            conn.close()
         elif msg.startswith(b'DEL_COMPUTE:'):
             msg = msg[len(b'DEL_COMPUTE:'):]
             try:
@@ -1063,9 +1089,18 @@ class _DispyNode(object):
         if self.scheduler['ip'] is None and self.avail_cpus == self.num_cpus:
             self.timer_coro.resume(None)
             Coro(self.broadcast_ping_msg)
-        if compute.cleanup is False:
+        if not compute.cleanup:
             return
         os.chdir(self.dest_path_prefix)
+        if isinstance(compute.cleanup, str):
+            try:
+                exec(marshal.loads(compute.code)) in globals(), locals()
+                _dispy_cleanup_func = locals()[compute.cleanup]
+                _dispy_cleanup_func()
+            except:
+                logger.debug('cleanup "%s" failed' % compute.cleanup)
+                logger.debug(traceback.format_exc())
+
         for xf in compute.xfer_files:
             tgt = os.path.join(compute.dest_path, os.path.basename(xf.name))
             if tgt not in self.file_uses:
